@@ -5,6 +5,8 @@
 #else
 #include <sys/types.h>
 #include <unistd.h>
+#include <stdio.h>
+#include <stdarg.h>
 #endif
 
 #include "mktmpdir.c"
@@ -120,7 +122,7 @@ char* shell_quote(const char *src)
      * add enclosing quotes and trailing \0 */
     char *dst = malloc(2 * strlen(src) + 3);
 
-    char *p = src;
+    const char *p = src;
     char *q = dst;
     char c;
 
@@ -130,19 +132,18 @@ char* shell_quote(const char *src)
     {
         if (c == '\\') 
         {
-            char *bs = p;               /* span of backslashes */
-            int n = strspn(bs, "\\");
+            int n = strspn(p, "\\");    /* span of backslashes starting at p */
 
-            memcpy(q, bs, n);           /* copy the span */
+            memcpy(q, p, n);            /* copy the span */
             q += n;
 
-            if (bs[n] == '\0' || bs[n] == '"')
+            if (p[n] == '\0' || p[n] == '"') /* span ends in quote or NUL */
             {
-                memcpy(q, bs, n);       /* copy the span once more */
+                memcpy(q, p, n);        /* copy the span once more */
                 q += n;
             }
 
-            p += n;
+            p += n;                     /* advance over the span */
             continue;
         }
 
@@ -159,11 +160,21 @@ char* shell_quote(const char *src)
 }
 #endif
 
+void die(const char *format, ...)
+{
+    va_list ap;
+    va_start(ap, format);
+    vfprintf(stderr, format, ap);
+    va_end(ap);
+    
+    exit(255);
+}
+
 char pp_version_info[] = "@(#) Packed by PAR::Packer " PAR_PACKER_VERSION;
 
 int main ( int argc, char **argv, char **env )
 {
-    int rc, i;
+    int rc;
     char *stmpdir;
     embedded_file_t *emb_file;
     char *my_file;
@@ -173,33 +184,30 @@ int main ( int argc, char **argv, char **env )
 typedef BOOL (WINAPI *pALLOW)(DWORD);
     HINSTANCE hinstLib;
     pALLOW ProcAdd;
+    char **argp;
 #ifndef ASFW_ANY
 #define ASFW_ANY -1
 #endif
 #endif
 
-#define DIE exit(255)
-
     par_init_env();
 
     stmpdir = par_mktmpdir( argv );	
-    if ( !stmpdir ) DIE;        /* error message has already been printed */
+    if ( !stmpdir ) die("");        /* error message has already been printed */
 
     rc = my_mkdir(stmpdir, 0700);
     if ( rc == -1 && errno != EEXIST) {
-	fprintf(stderr, "%s: creation of private cache subdirectory %s failed (errno= %i)\n", 
-                        argv[0], stmpdir, errno);
- 	DIE;
+	die("%s: creation of private cache subdirectory %s failed (errno= %i)\n", 
+            argv[0], stmpdir, errno);
     }
 
     /* extract embedded_files[0] (i.e. the custom Perl interpreter) 
      * into stmpdir (but under the same basename as argv[0]) */
-    my_prog = par_findprog(argv[0], strdup(par_getenv("PATH")));
+    my_prog = par_findprog(argv[0], par_getenv("PATH"));
     rc = extract_embedded_file(embedded_files, par_basename(my_prog), stmpdir, &my_perl);
     if (rc == EXTRACT_FAIL) {
-        fprintf(stderr, "%s: extraction of %s (custom Perl interpreter) failed (errno=%i)\n", 
-                            argv[0], my_perl, errno);
-        DIE;
+        die("%s: extraction of %s (custom Perl interpreter) failed (errno=%i)\n", 
+            argv[0], my_perl, errno);
     }
 
     if (rc == EXTRACT_OK)       /* i.e. file didn't already exist */
@@ -239,9 +247,8 @@ typedef BOOL (WINAPI *pALLOW)(DWORD);
     emb_file = embedded_files + 1;
     while (emb_file->name) {
         if (extract_embedded_file(emb_file, emb_file->name, stmpdir, &my_file) == EXTRACT_FAIL) {
-            fprintf(stderr, "%s: extraction of %s failed (errno=%i)\n", 
-                                argv[0], my_file, errno);
-            DIE;
+            die("%s: extraction of %s failed (errno=%i)\n", 
+                argv[0], my_file, errno);
         }
         emb_file++;
     }
@@ -261,7 +268,7 @@ typedef BOOL (WINAPI *pALLOW)(DWORD);
     par_setenv("PAR_SPAWNED", "1");
 
     /* quote argv strings if necessary, cf. Win32::ShellQuote */
-    for (char **argp = argv; *argp; argp++)
+    for (argp = argv; *argp; argp++)
     {
         int len = strlen(*argp);
         if ( len == 0 
@@ -278,7 +285,8 @@ typedef BOOL (WINAPI *pALLOW)(DWORD);
     exit(rc);
 #else
     execvp(my_perl, argv);
-    DIE;
+    die("%s: exec of %s (custom Perl interpreter) failed (errno=%i)\n", 
+        argv[0], my_perl, errno);
 #endif
 }
 
